#ifndef LTR24API_H_
#define LTR24API_H_

#include "ltrapi.h"

#ifdef __cplusplus
extern "C" {
#endif


#ifdef _WIN32
    #ifdef LTR24_EXPORTS
        #define LTR24_EXPORT(type) __declspec(dllexport) type APIENTRY
    #else
        #define LTR24_EXPORT(type) __declspec(dllimport) type APIENTRY
    #endif
#else
    #define LTR24_EXPORT(type) type
#endif



/* Текущая версия библиотеки (major, minor, build). */
#define LTR24_VERSION_CODE      0x01010000UL

/* Количество физических каналов. */
#define LTR24_CHANNEL_NUM       4
/* Количество диапазонов. */
#define LTR24_RANGE_NUM         2
/* Количество частот дискретизации. */
#define LTR24_FREQ_NUM          8

#define LTR24_NAME_SIZE         8

#define LTR24_SERIAL_SIZE       16

/* Коды ошибок. */
enum {
    LTR24_ERR_OPEN              = -10100,
    LTR24_ERR_NOT_OPEN          = -10101,
    LTR24_ERR_BUSY              = -10102,
    LTR24_ERR_MODULE_ID         = -10103,
    LTR24_ERR_SEND              = -10104,
    LTR24_ERR_RECV              = -10105,
    LTR24_ERR_TIMEOUT           = -10106,
    LTR24_ERR_NO_RESP           = -10107,
    LTR24_ERR_INVAL_RESP        = -10108,
    LTR24_ERR_STARTED           = -10109,
    LTR24_ERR_NOT_STARTED       = -10110,
    LTR24_ERR_INVAL_ARGS        = -10111,
    LTR24_ERR_INVAL_FREQ        = -10112,
    LTR24_ERR_INVAL_FORMAT      = -10113,
    LTR24_ERR_INVAL_CONFIG      = -10114,
    LTR24_ERR_INVAL_SYNC        = -10115,
    LTR24_ERR_INVAL_RANGE       = -10116,
    LTR24_ERR_WRONG_CRC         = -10117,
    LTR24_ERR_VERIFY_FAILED     = -10118,
    LTR24_ERR_DATA_FORMAT       = -10119,
    LTR24_ERR_UNALIGNED_DATA    = -10120,
    LTR24_ERR_DISCONT_DATA      = -10121,
    LTR24_ERR_CHANNELS_DISBL    = -10122,
    LTR24_ERR_UNSUP_VERS        = -10123,
    LTR24_ERR_FRAME_NOT_FOUND   = -10124,
    LTR24_ERR_OPEN_MCS_MOD      = -10125,
    LTR24_ERR_NO_SAVED_MCS      = -10126,
    LTR24_ERR_MCS_NOT_VALID     = -10127,
    LTR24_ERR_MCS_DIFF_MID      = -10128,
    LTR24_ERR_BAD_SYN_IO        = -10129,
    LTR24_ERR_LAST_             = -10130
};

/* Частоты дискретизации. */
enum {
    LTR24_FREQ_117, /* 117.1875 кГц     */
    LTR24_FREQ_78,  /* 78.125 кГц       */
    LTR24_FREQ_58,  /* 58.59375 кГц     */
    LTR24_FREQ_39,  /* 39.0625 кГц      */
    LTR24_FREQ_29,  /* 29.296875 кГц    */
    LTR24_FREQ_19,  /* 19.53125 кГц     */
    LTR24_FREQ_14,  /* 14.6484375 кГц   */
    LTR24_FREQ_9    /* 9.765625 кГц     */
};

/* Тип синхронизации. */
enum {
    LTR24_SYNC_NONE,    /* Без синхронизации.   */
    LTR24_SYNC_MASTER,  /* Ведущий модуль.      */
    LTR24_SYNC_SLAVE    /* Ведомый модуль.      */
};

/* Поддиапазоны измерения. */
enum {
    LTR24_RANGE_2,  /* +-2 В    */
    LTR24_RANGE_10  /* +-10 В   */
};

/* Разрядность данных. */
enum {
    LTR24_FORMAT_20,    /* 20-битный    */
    LTR24_FORMAT_24     /* 24-битный    */
};

#pragma pack(push, 4)

/* Информация о модуле. */
typedef struct {
    CHAR        Name[LTR24_NAME_SIZE];      /* Название модуля.         */
    CHAR        Serial[LTR24_SERIAL_SIZE];  /* Серийный номер модуля.   */
    BYTE        VerPLD;                     /* Версия прошивки ПЛИС.    */
    struct {
        float   Offset;
        float   Scale;
    }           CalibCoef[LTR24_CHANNEL_NUM][LTR24_RANGE_NUM][LTR24_FREQ_NUM];
                                            /* Заводские поправочные
                                             * коэффициенты.
                                             */
} TINFO_LTR24, *PTINFO_LTR24;

/* Управляющая структура модуля. */
typedef struct {
    INT         Size;           /* Размер структуры.                    */
    TLTR        Channel;        /* Канал связи с LTR сервером.          */
    BOOL        Run;            /* Сбор данных запущен.                 */
    BYTE        ADCFreqCode;    /* Код частоты дискретизации.           */
    double      ADCFreq;        /* Значени частоты дискретизации в Гц
                                 * (устанавливается после выполнения
                                 * LTR24_SetADC).
                                 */
    BYTE        DataFmt;        /* Разрядность данных.                  */
    BYTE        SyncMode;       /* Тип синхронизации.                   */
    BOOL        ZeroMode;       /* Режим измерения собственного нуля.   */
    struct {
        BOOL    Enable;         /* Канал включен.                       */
        BYTE    Range;          /* Код поддиапазона канала.             */
        BOOL    AC;             /* Режим отсечки постоянной составляющей.
                                 */
    }           ChannelMode[LTR24_CHANNEL_NUM]; /* Режимы канало.       */
    TINFO_LTR24 ModuleInfo;     /* Информация о модуле.                 */
    struct {
        float   Offset;
        float   Scale;
    }           CalibCoef[LTR24_CHANNEL_NUM][LTR24_RANGE_NUM][LTR24_FREQ_NUM];
                                /* Используемые поправочные коэффициенты.
                                 * По умолчанию записываются заводские
                                 * коэффициенты из флеш памяти.
                                 */
    PVOID       Reserved;       /* Зарезервировано для будущих
                                 * применений.
                                 */
} TLTR24, *PTLTR24;

#pragma pack(pop)



/* Возвращает текущую версию библиотеки. */
LTR24_EXPORT(DWORD) LTR24_GetVersion(void);

/* Инициализирует управляющую структуру. */
LTR24_EXPORT(INT) LTR24_Init(PTLTR24 hnd);

/* Открывает модуль в указанном крейте и слоте.
 * После открытия модуля автоматически считывается описание модуля из
 * флеш памяти и устанавливается режим работы по умолчанию.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_Open(PTLTR24 hnd, DWORD addr, WORD port, const CHAR *sn, INT slot);

/* Закрывает модуль. Если шел сбор данных, то он корректно останавливается.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_Close(PTLTR24 hnd);

/* Возвращает LTR_OK, если модуль открыт или код ошибки, если закрыт.
 */
LTR24_EXPORT(INT) LTR24_IsOpened(PTLTR24 hnd);

/* Считывает информацию из флеш памяти модуля и обновляет полу ModuleInfo в
 * управляющей структуре модуля.
 */
LTR24_EXPORT(INT) LTR24_GetConfig(PTLTR24 hnd);

/* Выполняет фазирование двух модулей. Используется только при работе в режиме
 * с синхронной работой нескольких модулей.
 * Для синхронного запуска необходимо правильно соединить модули линией
 * синхронизации. Далее выполнить следующую последовательность:
 * 1. Определить роли модулей, установить их режимы синхронизации в
 *    необходимые значения (ведущий или ведомый).
 * 2. Выполнить LTR24_SetADC на всех модулях.
 * 3. Выполнить LTR24_Phase на всех ведомых модулях.
 * 4. Выполнить LTR24_Phase на ведущем модуле.
 * 5. Выполнить LTR24_Start на всех ведомых модулях.
 * 6. Выполнить LTR24_Start на ведущем модуле.
 */
LTR24_EXPORT(INT) LTR24_Phase(PTLTR24 hnd);

/* Передает выбранные настройки в модуль.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_SetADC(PTLTR24 hnd);

/* Запускает сбор данных.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_Start(PTLTR24 hnd);

/* Останавливает сбор данных.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_Stop(PTLTR24 hnd);

/* Используется только при запущенном сборе данных.
 * Включает режим измерения собственного нуля.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_SetZeroMode(PTLTR24 hnd, BOOL enable);

/* Используется только при запущенном сборе данных.
 * Включает режим отсечения постоянной составляющей для каждого канала.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_SetACMode(PTLTR24 hnd, BYTE chan, BOOL ac_mode);

/* Принимает сырые данные от модуля.
 * Данные принимаются по-кадрово. Кадр содержит текушие значения по каждому
 * включенном каналу АЦП. С выключенных каналов данные не приходят.
 * Для 20-битного формата каждый отсчет АЦП состоит из одного
 * двойного слова (DWORD), для 24-битного формата - из двух двойных слов,
 * в которых передается отдельно старшая и младшая часть отсчета по данному
 * каналу.
 * Т.е. если количество включенных каналов равно 2, используется 20-битный
 * формат данных, то один кадр будет длиной 2 двойных слова или 8 байт. Для
 * 24-битного формата размер будет 4 двойных слова или 16 байт.
 * Данные возвращаются в массив data, метки времени - в массив tmark.
 * size - количество элементов массивов data и tmark.
 * timeout - максимальное время блокировки в функции для сбора данных.
 * Возвращается количество принятых двойных слов или код ошибки.
 */
LTR24_EXPORT(INT) LTR24_Recv(PTLTR24 hnd, DWORD *data, DWORD *tmark, DWORD size, DWORD timeout);

/* Обрабатывает сырые данные, принятые от модуля, применяет поправочные
 * коэффициенты, переводит в вольты или коды АЦП. Также проверяется
 * неразрывность принятых данных (в каждом формате есть счетчик слов).
 * size - количество элементов в массиве. После выполнения в size
 * записывается оставшееся количество элементов после удаления
 * некорректных данных (например, при передаче невыровненных по границе кадра
 * данных).
 * В случае передачи невыровненных по границе кадра данных функция вырезает
 * неполные кадры, производя вычисления над оставшимися данными, и возвращает
 * ошибку. Если входные данные имеют разрыв, то функция это также определяет
 * и возвращает ошибку.
 * В массив ovload возвращаются признаки перегрузки входных трактов каналов
 * (данный признак отслеживается только в 24-битном формате данных), количество
 * элементов массива ovload должен быть таким же, как и у массива dest
 * (т.е. на каждый отсчет возвращается его признак).
 * Поскольку при использовании 24-битного формата используется 2 двойных слова
 * на один отсчет, то количество элементов массива dest должно быть в 2 раза
 * меньше, чем у входного массива src.
 */
LTR24_EXPORT(INT) LTR24_ProcessData(PTLTR24 hnd, const DWORD *src, double *dest, INT *size,
                  BOOL calib, BOOL volt, BOOL *ovload);

/* Возвращает строку с описанием ошибки.
 */
LTR24_EXPORT(LPCSTR) LTR24_GetErrorString(INT err);

/* Сохраняет текущую урпавляющую структуру в крейт контроллере.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_StoreMcs(PTLTR24 hnd);

/* Восстанавливает управляющую структуру из крейт контроллера.
 * Должна вызываться после LTR24_Init и вместо LTR24_Open.
 */
LTR24_EXPORT(INT) LTR24_RestoreMcs(PTLTR24 hnd, DWORD addr, WORD port, const CHAR *serial, BYTE slot);

/* Очищает слот для хранения уравляющей структуры от сохраненных данных.
 * Возвращает код ошибки.
 */
LTR24_EXPORT(INT) LTR24_ClearMcsSlot(PTLTR24 hnd);

/* Определяет данные в слоте для хранения управляющей структуры
 * как некорректные.
 */
LTR24_EXPORT(INT) LTR24_InvalidateMcsSlot(PTLTR24 hnd);

/* Определяет данные в слоте для хранения управляющей структуры
 * как некорректные.
 */
LTR24_EXPORT(INT) LTR24_FindFrameStart(PTLTR24 hnd, const DWORD *data, INT size, INT *index);

/* Наладочные функции модуля. */

/* Тестирует выход SYN_OUT и вход SYN_IN. Если импульс от SYN_OUT не пришел
 * на SYN_IN, то возвращается ошибка. */
LTR24_EXPORT(INT) LTR24_TestSynOutAndSynIn(PTLTR24 hnd);

/* Записывает конфигурацию модуля во флеш. */
LTR24_EXPORT(INT)
LTR24_WriteConfig(PTLTR24 hnd);

#ifdef __cplusplus
}
#endif

#endif
