
#ifndef __LTR35API__
#define __LTR35API__

#ifdef _WIN32
    #ifdef LTR35API_EXPORTS
        #define LTR35API_DllExport(type)   __declspec(dllexport) type APIENTRY
    #else
        #define LTR35API_DllExport(type)   __declspec(dllimport) type APIENTRY
    #endif
#else
    #define LTR35API_DllExport(type) type
#endif


#include "ltrapi.h"


/** Коды ошибок, специфичные именно для модуля LTR35 */
typedef enum
{
    /** Невозможно открыть файл с прошивкой ПЛИС */
    LTR35_ERR_OPEN_FPGA_FIRM_FILE        = -10200,
    /** Превышен таймаут ожидания готовности ПЛИС к загрузке */
    LTR35_ERR_FPGA_NSTATUS_WAIT_TOUT     = -10201,
    /** Превышен таймаут ожидания перехода ПЛИС в рабочий режим */
    LTR35_ERR_FPGA_CONF_DONE_WAIT_TOUT   = -10202,
    /** Прошивка ПЛИС не загружена */
    LTR35_ERR_FPGA_IS_NOT_LOADED         = -10203
} t_ltr35_errs;


typedef enum
{
    LTR35_PREP_FLAGS_VOLT   = 0x01, /**< Флаг указывает, данные на входе заданны в вольтах
                                         и их нужно перевести в коды */
    LTR35_PREP_FLAGS_CALBIR = 0x02  /**< Флаг указывает, что к данным необходимо
                                         применить калибровочные коэффициенты */
} t_ltr35_prep_flags;

/** Количество каналов ЦАП */
#define LTR35_DAC_CHANNEL_CNT    8

/** Максимальное количество точек в странице ЦАП */
#define LTR35_MAX_POINTS_PER_PAGE    (4*1024*1024)

/** Диапазон каналов ЦАП */
#define LTR35_RANGE             10.

#pragma pack(4)
    //**** конфигурация модуля
    typedef struct
    {
        INT size;                    /**< Размер структуры */
        TLTR Channel;                /**< Описатель канала связи с модулем */
        void* internal;              /**< Указатель на внутреннее состояние модуля */
        DWORD res[128];              /**< Резерв */
    } TLTR35,*PTLTR35;
#pragma pack()

#ifdef __cplusplus
extern "C" {
#endif

    /** Инициализация полей структуры описателя модуля.
        Данную функцию необходимо вызывать перед использованием любых других функций
        с этим описателем модуля.
        @param[in] hnd   Описатель модуля
        @return          Код ошибки */
    LTR35API_DllExport(INT) LTR35_Init (TLTR35 *hnd);

    LTR35API_DllExport(INT) LTR35_Open (TLTR35 *hnd, DWORD net_addr, WORD net_port,
                                         const CHAR *crate_sn, WORD slot_num);
    // Разрыв связи с модулем.
    LTR35API_DllExport(INT) LTR35_Close (TLTR35 *hnd);

    // Определение сотояния канала связи с модулем.
    LTR35API_DllExport(INT) LTR35_IsOpened (TLTR35 *hnd);

    // функции вспомагательного характера
    LTR35API_DllExport(LPCSTR) LTR35_GetErrorString(INT err);

    /** Загрузка прошивки ПЛИС для модуля LTR35
        @param[in] hnd       Описатель модуля
        @param[in] filename  Имя файла с прошивкой ПЛИС
        @return              Код ошибки */
    LTR35API_DllExport(INT) LTR35_LoadFPGA(TLTR35* hnd, const char* filename);

    /** Проверка, загружена ли прошивка ПЛИС */
    LTR35API_DllExport(INT) LTR35_FPGAIsLoaded(TLTR35 *hnd);


    LTR35API_DllExport(INT) LTR35_Send(TLTR35 *hnd, const DWORD *data,
                                        DWORD size, DWORD timeout);

    LTR35API_DllExport(INT) LTR35_PrepareData(TLTR35 *hnd, const double *in_data,
                                                DWORD *result, DWORD size, DWORD flags);

    LTR35API_DllExport(INT) LTR35_SwitchDacPage(TLTR35 *hnd, DWORD flags);


#ifdef __cplusplus
}
#endif

#endif

